'use strict';

window.angular.module('BillyCmsBackend').controller('PageTypesCtrl', ['$scope', '$http', 'PageType', function($scope, $http, PageType) {
    var self = this;
    $scope.$watch(
        function() { return $scope.app.allPageTypes; },
        function(newPageTypes, oldPageTypes) {
            if ((oldPageTypes === undefined || !oldPageTypes.length) && newPageTypes && newPageTypes.length) {
                self.selectPageType(newPageTypes[0]);
            }
        }
    );

    $scope.$watch('ptctrl.attributeIdToAdd', function(nextVal) {
        if (nextVal) {
            self.addAtributeToSelectedPageType();
        }
    });

    this.selectPageType = function(pageType) {
        this.selectedPageType = pageType;
    };

    if ($scope.app.allPageTypes && $scope.app.allPageTypes.length > 0) {
        this.selectPageType($scope.app.allPageTypes[0]);
    }

    this.addAtributeToSelectedPageType = function() {
        var attributeId = this.attributeIdToAdd;
        var pageType = this.selectedPageType;
        if (!pageType) {
            return console.error('No Page Type selected.');
        }
        return pageType
            .addAdditionalAttribute(attributeId)
            .then(function() {
                self.attributeIdToAdd = null;
            })
            .catch(function(error) {
                alert(error);
            });
    };

    this.newPageType = new PageType({});

    this.selectedTabIndex = 0;

    this.resetNewPageType = function() {
        this.newPageType = new PageType();
    };
    this.resetNewPageType();

    this.createPageType = function() {
        $http.post('/billy_cms/api/page_types.json', this.newPageType)
            .then(function(result) {
                var pageType = new PageType(result.data.page_type);
                $scope.app.allPageTypes.push(pageType);

                self.selectedTabIndex = 0;
            })
            .catch(function(err) {
                alert(err);
            })
    };

    this.removeAttribute = function(attribute) {
        return this.selectedPageType.removeAdditionalAttribute(attribute).catch(function(err) {
            alert(err);
        });
    };

    this.attributesAddableToSelectedPageType = function() {
        if (!self.selectedPageType) {
            return [];
        }
        return ($scope.app.allAdditionalAttributes || []).filter(function(attribute) {
            return self.selectedPageType.additional_attributes
                .map(function(attr) { return attr.id; })
                .indexOf(attribute.id) < 0;
        });
    }
}]);