'use strict';

angular.module('BillyCmsBackend').factory('PageType', ['AdditionalAttribute', '$http', function(AdditionalAttribute, $http) {
    var PageType = function(options) {
        if (!options) { options = {} }
        this.applyOptions(options);
    };

    PageType.prototype.applyOptions = function(options) {
        this.id = options.id || null;
        this.name = options.name || null;
        this.title = options.title || '';

        this.additional_attributes = (options.additional_attributes || []).map(function(attributeOptions) {
            return new AdditionalAttribute(attributeOptions);
        })
    }

    PageType.prototype.addAdditionalAttribute = function(attributeOrAttributeId) {
        var attributeId;
        var self = this;
        if (!this.id) {
            throw new Error('PageType is not valid.');
        }
        if (typeof attributeOrAttributeId === 'number' || typeof attributeOrAttributeId === 'string') {
            attributeId = parseInt(attributeOrAttributeId, 10);
        } else {
            attributeId = attributeOrAttributeId.id;
        }
        if (!attributeId) {
            throw new Error('Please pass a valid attribute or attribute id.');
        }
        return $http.post('/billy_cms/api/page_types/' + String(this.id) + '/attributes.json', {
            attribute: {
                id: attributeId
            }
        }).then(function(result) {
            self.applyOptions(result.data.page_type)
        });
    }

    PageType.prototype.removeAdditionalAttribute = function(attributeOrAttributeId) {
        var self = this;
        var attributeId;
        if (!this.id) {
            throw new Error('PageType is not valid.');
        }
        if (typeof attributeOrAttributeId === 'number' || typeof attributeOrAttributeId === 'string') {
            attributeId = parseInt(attributeOrAttributeId, 10);
        } else {
            attributeId = attributeOrAttributeId.id;
        }
        if (!attributeId) {
            throw new Error('Please pass a valid attribute or attribute id.');
        }
        return $http.delete('/billy_cms/api/page_types/' + String(this.id) + '/attributes/' + String(attributeId))
            .then(function(result) {
                self.applyOptions(result.data.page_type)
            });
    }

    return PageType;

}]);
