'use strict';

var BillyCms = window.BillyCms || {};

BillyCms.API_BASE_URL = '/billy_cms/api/pages';

BillyCms.Editing = {
    isEditModeOn: false,

    setEditMode(state, element) {
        this.isEditModeOn = state;
        var editableFields;
        if (element) {
            editableFields = $(element).parents('.page-wrapper').first().find('[contenteditable]');
        } else {
            editableFields = $('[contenteditable]');
        }
        var container = $(document.body);
        if (state === true) {
            container.addClass('edit-mode-enabled');
            editableFields.attr('contenteditable', true);
        } else {
            container.removeClass('edit-mode-enabled');
            editableFields.attr('contenteditable', false);
        }
    },

    savePageProperty(page_id, property, content) {
        return $.ajax({
            url: `${BillyCms.API_BASE_URL}/${page_id}`,
            method: 'PATCH',
            data: {
                page: {
                    [property]: content
                }
            }
        }).done(data => {
            if (data.success) {
                console.log('page saved.');
                console.log(data.page);
            } else {
                alert(`Es hat sich ein Fehler ereignet: ${data.error}`);
            }
        }).fail((jqXHR, textStatus, error) => {
            alert(`Es hat sich ein Fehler ereignet: ${error}`);
        });
    }
};

$(function() {
    $('.billycms_edit_button').on('click', function() {
        BillyCms.Editing.setEditMode(!BillyCms.Editing.isEditModeOn, this);
    });

    $('.page-content-binary input[type=file]').on('change', function(evt) {
        if (BillyCms.Editing.isEditModeOn) {
            var container = $(this).parent('.page-content-binary');
            var pageId = parseInt(container.attr('billycms-page'), 10);
            var files = this.files;
            if (files.length < 1) { return; }
            var file = files[0];
            var reader = new FileReader();
            var image = container.find('img');

            reader.onloadend = function() {
                var result = reader.result;
                BillyCms.Editing.savePageProperty(pageId, 'content', result)
                    .done(function() {
                        image.attr('src', result);
                    })
            };
            reader.readAsDataURL(file);
        }
    });

    $('[contenteditable]')
        .on('blur', function() {
            if (BillyCms.Editing.isEditModeOn) {
                let el = $(this),
                    id = el.attr('billycms-page'),
                    attribute = el.attr('billycms-attribute');
                BillyCms.Editing.savePageProperty(id, attribute, el.html())
            }
        })
        .on('paste', e => {
            e.preventDefault();
            let text = e.originalEvent.clipboardData.getData('text/plain');
            document.execCommand("insertHTML", false, text);
        });

    $('.page-settings-modal').each((index, modalElement) => {
        let pageSettingsModal = $(modalElement);
        let pageSettingsForm = pageSettingsModal.find('form').first();
        let pageSettingsButton = pageSettingsModal.find('.btn.btn-primary').last();
        let saveSettingsFn = function() {
            let form = pageSettingsForm;
            let id = form.attr('billycms-id');
            if (!id) {
                return alert('Diese Seite existiert nicht.');
            }
            id = form.attr('billycms-id');
            let page = {};
            form.find('input').not('[type=hidden]').not('[disabled]').each((i, element) => {
                let el = $(element);
                let name = el.attr('name');
                let val = el.val();
                if (name) {
                    if (val === 'true') {
                        page[name] = true;
                    } else if (val === 'false') {
                        page[name] = false;
                    } else {
                        page[name] = val;
                    }
                }
            });
            console.log(page);
            $.ajax({
                url: `${BillyCms.API_BASE_URL}/${id}`,
                method: 'PATCH',
                data: { page }
            }).done(data => {
                if (data.success) {
                    console.log('page saved.');
                    console.log(data.page);
                    pageSettingsModal.modal('hide');
                } else {
                    alert(`Es hat sich ein Fehler ereignet: ${data.error}`);
                }
            }).fail((jqXHR, textStatus, error) => {
                alert(`Es hat sich ein Fehler ereignet: ${error}`);
            });
        };
        pageSettingsButton.on('click', saveSettingsFn);
        pageSettingsForm.on('submit', saveSettingsFn);
    });
});
