module Api
    class AttributesController < ApiController
        def index
            render json: BillyCms::AdditionalAttribute.all
        end

        def show
            render json: BillyCms::AdditionalAttribute.find(params[:id])
        end

        def update
            render nothing: true, status: 400 unless params[:attribute]
            attribute = BillyCms::AdditionalAttribute.find params[:id]
            if attribute.update_attributes attributes_params
                render json: {
                    success: true,
                    attribute: attribute
                }
            else
                render json: {
                    success: false,
                    error: attribute.error_messages,
                }, status: 500
            end
        end

        def create
            render nothing: true, status: 400 unless params[:attribute]
            attribute = BillyCms::AdditionalAttribute.new attributes_params
            if attribute.save
                render json: {
                    success: true,
                    attribute: attribute
                }
            else
                render json: {
                    success: false,
                    error: attribute.errors.full_messages.join('. '),
                }, status: 500
            end
        end

        def destroy
            render :nothing, status: 400 unless params[:id]
            if BillyCms::AdditionalAttribute.delete(params[:id])
                render json: {
                    success: true
                }
            else
                render json: {
                    success: false,
                    error: attribute.error_messages,
                }, status: 500
            end
        end

        private

        def attributes_params
            params.require(:attribute).permit(:title, :name, :attribute_type) if current_user.admin?
        end
    end
end
