module Api
  class PageTypeAttributesController < ApiController
    before_filter :set_page_type
    before_filter :set_additional_attribute, only: [:create]
    before_filter :ensure_user_is_admin

    def index
      render json: @page_type.additional_attributes
    end

    def show
      render json: @page_type.additional_attributes.find(params.fetch(:id))
    end

    def create
      if @page_type && @additional_attribute
        if @page_type.additional_attributes.include? @additional_attribute
          render json: { success: false, error: 'Attribute already exists.' }, status: 400
        else
          @page_type.additional_attributes.push @additional_attribute
          render json: { success: true, page_type: BillyCms::PageTypeSerializer.new(@page_type) }
        end
      end
    end

    def destroy
      additional_attribute_id = params[:id]
      @additional_attribute = BillyCms::AdditionalAttribute.find(additional_attribute_id)
      if @page_type && @additional_attribute
        unless @page_type.additional_attributes.include? @additional_attribute
          render json: { success: false, error: "Attribute does not exist on pages of type '#{@page_type.name}'." }, status: 400
        else
          @page_type.additional_attributes.delete(additional_attribute_id)
          render json: { success: true, page_type: BillyCms::PageTypeSerializer.new(@page_type) }
        end
      end
    end

    private

    def set_page_type
      @page_type = BillyCms::PageType.find(params.fetch(:page_type_id))
    end

    def set_additional_attribute
      if id = attribute_params[:id]
        begin
          @additional_attribute = BillyCms::AdditionalAttribute.find id
        rescue ActiveRecord::RecordNotFound
          @additional_attribute = nil
        end
      end
    end

    def attribute_params
      params.require(:attribute).permit(:id)
    end
  end
end
