module Api
  class PagesController < ApiController
    def index
      render json: BillyCms::Page.all
    end

    def show
      render json: BillyCms::Page.find(params[:id])
    end

    def update
      return render nothing: true, status: 400 unless params[:page]
      page = BillyCms::Page.find params[:id]
      if page.update_attributes page_params
        render json: {
            success: true,
            page: page
        }
      else
        render json: {
            success: false,
            error: page.errors.full_messages,
        }, status: 500
      end
    end

    def create
      return render nothing: true, status: 400 unless params[:page]
      page =  BillyCms::Page.new page_params
      if page.save
        render json: {
            success: true,
            page: page
        }
      else
        render json: {
            success: false,
            error: page.error_messages,
        }, status: 500
      end
    end

    def destroy
      return render :nothing, status: 400 unless params[:id]
      return render :nothing, status: 404 unless page = BillyCms::Page.where(id: params[:id]).first
      if page.destroy
        render json: {
            success: true
        }
      else
        render json: {
            success: false,
            error: page.error_messages,
        }, status: 500
      end
    end

    private

    def page_params
      page = BillyCms::Page.find(params[:id]) if params[:id].present?
      allowed_types = page ? page.additional_attributes.map(&:name).map(&:to_sym) : []
      params.require(:page).permit(:title, :content, :parent_page_id, :order_key, :hide_from_navigation, :page_type, additional_attributes_values: allowed_types) if current_user.admin?
    end
  end
end
