require 'friendly_id'

module BillyCms
  class Page < ActiveRecord::Base
    include ActiveModel::Serialization
    
    extend FriendlyId
    friendly_id :title, use: :slugged

    has_many :pages, class_name: 'Page', foreign_key: :parent_page_id, :dependent => :destroy
    belongs_to :parent_page, class_name: 'Page', foreign_key: :parent_page_id, optional: true

    def cms_path
      get_cms_path_component_for_parent '', self
    end

    def children
      pages.sort_by(&:order_key)
    end

    def navigation
      self.children.reject { |page| page.hide_from_navigation? || page.page_type == 'Binary' || page.page_type == 'box' }
    end

    def boxes
        pages.where(page_type: 'box').sort_by(&:order_key)
    end

    def binaries
        pages.where(page_type: 'Binary').sort_by(&:order_key)
    end

    def is_in_path?(path)
      if path == '/'
        return slug == 'startseite' ? true : false
      end
      # byebug if id == 6
      reg = Regexp.new("^(#{cms_path})", 'i')
      path.match(reg) != nil
    end

    def sibblings
      self.class.where(parent_page_id: parent_page_id).where.not(id: id).order(:order_key)
    end

    def binary?
        self.page_type == 'Binary'
    end

    def additional_attributes
        type = PageType.find_by_name(self.page_type)
        return [] unless type.present?
        type.additional_attributes
    end

    def additional_attribute(attribute_name)
        self.additional_attributes_values[attribute_name.to_s] if self.additional_attributes_values.present?
    end

    private
      def get_cms_path_component_for_parent(current_path, current_page)
        current_path += "#{get_cms_path_component_for_parent(current_path, current_page.parent_page) if current_page.parent_page}#{"/#{current_page.slug}" if current_page}"
      end
  end
end
